"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const discord_js_1 = require("discord.js");
const Commands_1 = require("../Commands");
const Client_types_1 = require("./Client.types");
const fs_1 = __importDefault(require("fs"));
const Debug_1 = require("../Debug");
const utils_1 = require("../utils");
const path_1 = require("path");
const utils_2 = require("../utils");
const Client_presence_1 = require("./Client.presence");
class Client {
    clientId = '';
    clientToken = '';
    commandsPath = '';
    eventsPath = '';
    intents = [];
    partials = [];
    presences = {
        activities: [],
        presenceUpdateFrequency: 0,
        updateOrder: Client_types_1.EPresenceUpdateStatus.OriginalOrder
    };
    commands = [];
    events = [];
    /**
     * Crear nueva instancia de un cliente Discord
     * @param {IClient} options - Opciones para crear nueva instancia de un nuevo cliente
     */
    constructor(options) {
        //. Asignación de propiedades privadas por medio de objeto en constructor
        if (typeof options === 'object') {
            if ('clientId' in options && typeof options.clientId === 'string' && !(0, utils_2.stringPropertyAreValid)(options.clientId))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.ClientId);
            if ('token' in options && typeof options.token === 'string' && !(0, utils_2.stringPropertyAreValid)(options.token))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.ClientToken);
            if ('eventsPath' in options && typeof options.eventsPath === 'string' && !(0, utils_2.stringPropertyAreValid)(options.eventsPath))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.CommandsPath);
            if ('commandsPath' in options && typeof options.commandsPath === 'string' && !(0, utils_2.stringPropertyAreValid)(options.commandsPath))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.EventsPath);
            if ('intents' in options && options.intents && !this.areIntentsValid(options.intents))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.Intents);
            if ('partials' in options && options.partials && !this.arePartialsValid(options.partials))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.Partials);
            if ('presence' in options && options.presence && !this.arePresenceValid(options.presence))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.Presence);
            if ('presenceUpdateFrequency' in options && options.presenceUpdateFrequency && typeof options.presenceUpdateFrequency !== 'number')
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.PresenceUpdateFrequency);
            if ('presenceUpdateChangeOrder' in options && options.presenceUpdateChangeOrder && !Object.keys(Client_types_1.EPresenceUpdateStatus).includes(options.presenceUpdateChangeOrder))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.PresenceUpdateStatus);
            if (options?.clientId)
                this.clientId = options.clientId;
            if (options?.token)
                this.clientToken = options.token;
            if (options?.commandsPath)
                this.commandsPath = options.commandsPath;
            if (options?.eventsPath)
                this.eventsPath = options.eventsPath;
            if (options?.intents)
                this.intents = options.intents;
            if (options?.partials)
                this.partials = options.partials;
            if (options?.presence)
                this.presences.activities = [options.presence].flatMap(x => x);
            if (options?.presenceUpdateFrequency)
                this.presences.presenceUpdateFrequency = options.presenceUpdateFrequency;
            if (options?.presenceUpdateChangeOrder)
                this.presences.updateOrder = options.presenceUpdateChangeOrder;
            this.start();
        }
    }
    //. Asignación de propiedades privadas por medio de métodos
    /**
     * Establecer ID del cliente Discord
     * @param clientId ID del cliente Discord
     */
    setClientId(clientId) {
        if (!(0, utils_2.stringPropertyAreValid)(clientId))
            return (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.ClientId);
        this.clientId = clientId;
        return this;
    }
    /**
     * Establecer Token del cliente Discord
     * @param token Token del cliente Discord
     */
    setToken(token) {
        if (!(0, utils_2.stringPropertyAreValid)(token))
            return (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.ClientToken);
        this.clientToken = token;
        return this;
    }
    /**
     * Establecer rutas para comandos
     * @param path Ruta de los comandos
     */
    setCommandsPath(path) {
        if (!(0, utils_2.stringPropertyAreValid)(path))
            return (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.EventsPath);
        this.commandsPath = path;
        return this;
    }
    /**
     * Establecer rutas para eventos
     * @param path Ruta de los eventos
     */
    setEventsPath(path) {
        if (!(0, utils_2.stringPropertyAreValid)(path))
            return (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.CommandsPath);
        this.eventsPath = path;
        return this;
    }
    /**
     * Establecer intents del cliente Discord.
     * @param intents Arreglo de intents del cliente Discord
     */
    setIntents(intents) {
        if (!this.areIntentsValid(intents))
            return (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.Intents);
        this.intents = intents;
        return this;
    }
    /**
     * Establecer partials del cliente Discord
     * @param partials - Arreglo de partials del cliente Discord
     * @returns
     */
    setPartials(partials) {
        if (!this.arePartialsValid(partials))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.Partials);
        this.partials = partials;
        return this;
    }
    /**
     * Establecer presence dinámicos para el cliente Discord
     * @param options Opciones para crear presence dinámicos para el cliente Discord
     */
    setPresence(options) {
        const { presenceActivities, presenceUpdateFrequency = 10000, updateOrder = Client_types_1.EPresenceUpdateStatus.OriginalOrder } = options;
        if (!this.arePresenceValid(presenceActivities))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.Presence);
        if (typeof presenceUpdateFrequency != 'number')
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.PresenceUpdateFrequency);
        if (Object.keys(Client_types_1.EPresenceUpdateStatus).includes(updateOrder))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Client.PresenceUpdateStatus);
        const activities = !Array.isArray(presenceActivities) ? [presenceActivities] : presenceActivities;
        const activitiesWithId = activities.map(activity => ({ ...activity, _id: Symbol('activity_id') }));
        this.presences = { activities: activitiesWithId, presenceUpdateFrequency, updateOrder };
        return this;
    }
    /**
     * Iniciar ejecución posterior a la configuración del cliente
     */
    async start() {
        (0, Debug_1.debugInfo)('Iniciando...');
        await this._loadCommands();
        this._loadEvents();
        (0, Debug_1.debugBot)(`[${(0, Debug_1.highlight)(this.intents.length)}] intent(s) cargado(s) | [${(0, Debug_1.highlight)(this.partials.length)}] partial(s) cargado(s)`);
        (0, Debug_1.debugBot)(`[${(0, Debug_1.highlight)(this.presences.activities.length)}] presence(s) cargados | cambio automático: ${(0, Debug_1.highlight)(`${this.presences.presenceUpdateFrequency / 1000}s`)}`);
        const client = new discord_js_1.Client({ intents: this.intents, partials: this.partials });
        if (this.events.length > 0)
            for (const event of this.events) {
                client.on(event.name, (...args) => event.execute(client, ...args));
            }
        client.on(discord_js_1.Events.InteractionCreate, interaction => {
            if (!interaction.isChatInputCommand())
                return;
            const command = this.commands.find(x => x.name === interaction.commandName);
            if (command)
                command.execute(client, interaction);
        });
        client.on(discord_js_1.Events.ClientReady, (client) => {
            (0, Debug_1.debugBot)(`Iniciado satisfactoriamente como ${Debug_1.ETerminalColors.BrightYellow}${client.user.tag}${Debug_1.ETerminalColors.Reset}`);
            (0, Debug_1.debugSuccess)('Carga finalizada.');
        });
        await client.login(this.clientToken).then(() => {
            if (this.presences.activities.length > 0)
                (0, Client_presence_1.startPresenceService)(client, this.presences);
        }).catch((reason) => {
            if (typeof reason !== 'string')
                reason = JSON.stringify(reason, null, 2);
            (0, Debug_1.debugError)(`Problema al iniciar el servicio del bot: ${reason}`);
        });
    }
    //* Cargadores
    async _loadCommands() {
        (0, Debug_1.debugCommand)(`Iniciando carga de comandos...`);
        if (this.commandsPath.length === 0)
            (0, Debug_1.debugCommand)('No se ha establecido una ruta de comandos');
        if (!fs_1.default.existsSync(this.commandsPath))
            (0, Debug_1.debugError)(`La ruta establecida para comandos es inválida: ${this.commandsPath}`);
        const slashCommandsBody = [];
        if (this.commandsPath.length > 0) {
            const commandsLoaded = (0, utils_1.getFiles)(this.commandsPath, ['.js', '.ts']);
            if (commandsLoaded.length > 0) {
                for (const commandLoaded of commandsLoaded) {
                    const filename = (0, path_1.basename)(commandLoaded);
                    if (filename.endsWith('.ignore.ts') || filename.endsWith('ignore.js')) {
                        (0, Debug_1.debugCommand)(`ignorado: ${(0, Debug_1.highlight)(filename, Debug_1.ETerminalColors.BrightCyan)}`);
                        continue;
                    }
                    const filepath = (0, path_1.dirname)(commandLoaded);
                    const requiredFile = require((0, path_1.join)(filepath, filename));
                    const command = requiredFile?.default ? requiredFile.default : requiredFile;
                    (0, utils_2.validateCommandOptions)(command);
                    (0, Debug_1.debugCommandLoaded)(command);
                    const slashCommand = (0, Commands_1.createSlashCommand)(command);
                    slashCommandsBody.push(slashCommand.toJSON());
                    this.commands.push(command);
                }
                (0, Debug_1.debugCommand)('Todos los comandos han sido cargados');
            }
            else
                (0, Debug_1.debugCommand)('No se han encontrado archivos de comandos');
        }
        (0, Debug_1.debugCommand)(`Actualizando slash commands...`);
        const rest = new discord_js_1.REST({ version: '10' }).setToken(this.clientToken);
        const slashCommandsUpdateStatus = await rest.put(discord_js_1.Routes.applicationCommands(this.clientId), { body: slashCommandsBody }).catch((e) => e);
        if (slashCommandsUpdateStatus === false)
            (0, Debug_1.debugError)('Ha ocurrido un error al actualizar los slash commands. Verifica los comandos');
        (0, Debug_1.debugCommand)('Los slash commands han sido actualizados correctamente');
    }
    _loadEvents() {
        (0, Debug_1.debugEvent)('Iniciando carga de eventos...');
        if (this.eventsPath.length === 0)
            return (0, Debug_1.debugEvent)('No se ha establecido una ruta de eventos');
        if (!fs_1.default.existsSync(this.eventsPath))
            (0, Debug_1.debugError)(`La ruta establecida para eventos es inválida: ${this.eventsPath}`);
        const eventsLoaded = (0, utils_1.getFiles)(this.eventsPath, ['.js', '.ts']);
        if (eventsLoaded.length === 0)
            return (0, Debug_1.debugEvent)('No se han encontrado archivos de eventos en la ruta asignada');
        for (const eventLoaded of eventsLoaded) {
            const filepath = (0, path_1.dirname)(eventLoaded);
            const filename = (0, path_1.basename)(eventLoaded);
            if (filename.endsWith('.ignore.ts') || filename.endsWith('ignore.js')) {
                (0, Debug_1.debugEvent)(`ignorado: ${(0, Debug_1.highlight)(filename, Debug_1.ETerminalColors.BrightCyan)}`);
                continue;
            }
            const fileRequired = require((0, path_1.join)(filepath, filename));
            const event = fileRequired?.default ? fileRequired.default : fileRequired;
            (0, Debug_1.debugEventLoaded)(event);
            this.events.push(event);
        }
        (0, Debug_1.debugEvent)('Todos los eventos han sido cargados');
    }
    //. Validaciones simples de la creación de cliente
    areIntentsValid(intents) {
        const validIntents = Object.values(discord_js_1.GatewayIntentBits);
        if (!Array.isArray(intents) || !intents.some(i => validIntents.includes(i)))
            return false;
        return true;
    }
    arePartialsValid(partials) {
        const validPartials = Object.values(discord_js_1.Partials);
        if (!Array.isArray(partials) || !partials.some(p => validPartials.includes(p)))
            return false;
        return true;
    }
    arePresenceValid(presenceList) {
        const validPresenceStatus = Object.values(discord_js_1.PresenceUpdateStatus);
        const validPresenceActivity = Object.values(discord_js_1.ActivityType);
        if (!Array.isArray(presenceList) && typeof presenceList !== 'object' && presenceList === null)
            return false;
        presenceList = [presenceList].flatMap(x => x);
        for (const presence of presenceList) {
            if (typeof presence.status === 'string' && !validPresenceStatus.includes(presence.status))
                return false;
            if (typeof presence.type === 'string' && !validPresenceActivity.includes(presence.type))
                return false;
        }
        return true;
    }
}
exports.default = Client;
