"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSlashCommand = void 0;
const discord_js_1 = require("discord.js");
const Debug_1 = require("../Debug");
const utils_1 = require("../utils");
class Command {
    _name = '';
    _description = '';
    _options = [];
    _subcommands = [];
    _subcommandsGroup = [];
    _isNSFW = false;
    _execute = () => null;
    //.Getters Start
    get name() {
        return this._name;
    }
    get description() {
        return this._description;
    }
    get options() {
        return this._options;
    }
    get subcommands() {
        return this._subcommands;
    }
    get subcommandsGroup() {
        return this._subcommandsGroup;
    }
    get isNSFW() {
        return this._isNSFW;
    }
    get execute() {
        return this._execute;
    }
    //. Getters End
    /**
     * Crear nueva instancia de un nuevo comando para el cliente Discord
     * @param options Opciones para crear un nuevo bot
     */
    constructor(options) {
        if (typeof options === 'object') {
            this.validateCommandOptions(options);
            if ('name' in options && typeof options.name === 'string')
                this._name = options.name;
            if ('description' in options && typeof options.description === 'string')
                this._description = options.description;
            if ('isNSFW' in options)
                this._isNSFW = options.isNSFW ?? false;
            if ('execute' in options)
                this._execute = options.execute;
            if ('options' in options)
                this._options = [options.options ?? []].flatMap(x => x);
            if ('subcommands' in options)
                this._subcommands = [options.subcommands ?? []].flatMap(x => x);
            if ('subcommandsGroup' in options)
                this._subcommandsGroup = [options.subcommandsGroup ?? []].flatMap(x => x);
        }
    }
    /**
     * Establecer nombre del comando
     * @param commandName Nombre del comando
     */
    setName(commandName) {
        if (!(0, utils_1.stringPropertyAreValid)(commandName) || /(A-Z)/gi.test(commandName) || commandName.includes(' '))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Name);
        this._name = commandName;
        return this;
    }
    /**
     * Establecer descripción del comando
     * @param commandDescription Descripción del comando
     */
    setDescription(commandDescription) {
        if (!(0, utils_1.stringPropertyAreValid)(commandDescription))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Description(this._name));
        this._description = commandDescription;
        return this;
    }
    /**
     * Establecer opciones para el comando
     * @param options Opciones del comando
     */
    setOptions(options) {
        options = [options].flatMap(x => x);
        if (!options.some(o => (0, utils_1.isInstanceOfTCommandOptions)(o)))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Options(this._name));
        this._options = options;
        return this;
    }
    /**
     * Establecer subcomandos al comando
     * @param subcommands Arreglo de Subcomandos
     */
    setSubcommands(subcommands) {
        subcommands = [subcommands].flatMap(x => x);
        if (!subcommands.some(s => s instanceof discord_js_1.SlashCommandSubcommandBuilder))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Subcommand(this._name));
        this._subcommands = subcommands;
        return this;
    }
    /**
     * Establecer grupo de subcomandos
     * @param subcommandsGroup Arreglo de grupo de subcommandos
     */
    setSubcommandsGroup(subcommandsGroup) {
        subcommandsGroup = [subcommandsGroup].flatMap(x => x);
        if (!Array.isArray(subcommandsGroup) || !subcommandsGroup.some(s => s instanceof discord_js_1.SlashCommandSubcommandGroupBuilder))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.SubcommandGroup(this._name));
        this._subcommandsGroup = subcommandsGroup;
        return this;
    }
    /**
     * Método para establecer la función a ejecutar cuando el comando sea invocado
     * @param executable Función a ejecutar
     */
    setExecutable(executable) {
        if (typeof executable !== 'function')
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Executable(this._name));
        this._execute = executable;
        return this;
    }
    /**
     * Establecer si el comando es NSFW
     * @param isNSFW Estado NSFW del comando
     */
    setIsNSFW(isNSFW) {
        if (typeof isNSFW !== 'boolean')
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.NSFW(this._name));
        this._isNSFW = isNSFW;
        return this;
    }
    //. Validadores simples
    validateCommandOptions(options) {
        if ('name' in options && (!(0, utils_1.stringPropertyAreValid)(options.name) || options.name.includes(' ') || /(A-Z)/gi.test(options.name)))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Name);
        if ('description' in options && !(0, utils_1.stringPropertyAreValid)(options.description))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Description(this._name));
        if ('options' in options && options.options && (!Array.isArray(options.options) || options.options.some(o => !(0, utils_1.isInstanceOfTCommandOptions)(o))))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Options(this._name));
        if ('subcommands' in options && options.subcommands && (!Array.isArray(options.subcommands) || options.subcommands.some(s => s instanceof discord_js_1.SlashCommandSubcommandBuilder)))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Subcommand(this._name));
        if ('subcommandsGroup' in options && options.subcommandsGroup && (!Array.isArray(options.subcommandsGroup) || options.subcommandsGroup.some(s => s instanceof discord_js_1.SlashCommandSubcommandGroupBuilder)))
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.SubcommandGroup(this._name));
        if ('isNSFW' in options && typeof options.isNSFW !== 'boolean')
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.NSFW(this._name));
        if ('executable' in options && typeof options.executable !== 'function')
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Executable(this._name));
    }
}
exports.default = Command;
const createSlashCommand = (command) => {
    const slashCommand = new discord_js_1.SlashCommandBuilder()
        .setName(command.name)
        .setDescription(command.description)
        .setNSFW(command.isNSFW);
    if (command.options.length > 0)
        for (const commandOption of command.options) {
            if (commandOption instanceof discord_js_1.SlashCommandAttachmentOption)
                slashCommand.addAttachmentOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandStringOption)
                slashCommand.addStringOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandBooleanOption)
                slashCommand.addBooleanOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandChannelOption)
                slashCommand.addChannelOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandIntegerOption)
                slashCommand.addIntegerOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandMentionableOption)
                slashCommand.addMentionableOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandNumberOption)
                slashCommand.addNumberOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandRoleOption)
                slashCommand.addRoleOption(commandOption);
            else if (commandOption instanceof discord_js_1.SlashCommandUserOption)
                slashCommand.addUserOption(commandOption);
        }
    if (command.subcommands.length > 0)
        for (const subcommand of command.subcommands) {
            slashCommand.addSubcommand(subcommand);
        }
    if (command.subcommandsGroup.length > 0)
        for (const subcommandGroup of command.subcommandsGroup) {
            slashCommand.addSubcommandGroup(subcommandGroup);
        }
    return slashCommand;
};
exports.createSlashCommand = createSlashCommand;
